-----------------------------------------------------------------------------
--! @file Stepper_IF.vhd
--! @author  Prof. Dr.-Ing. Ferdinand Englberger
--! @author  Prof. Dr.-Ing. Thomas Latzel
--! @version V1.0
--! @date    16.12.2010
--!
--! @brief Interface Schrittmotor ohne Incremental Encoder
--! @details Definition des Interfaces\n
--! @verbatim
--!   Name        ADDR  Breite  Zugriff  Aufbau
--! control_reg    0      7       R/W    IR IE Start/Stop Mode(3 Bit) Left/Right
--! speedstep_reg  1      4       R/W    MSB ... LSB  (0 - 15)
--! steps_reg      2     14       R/W    MSB ... LSB  (0 - 9999)
--! @endverbatim
--! @details Die Inhalte der Register @b control und @b speedstep werden auf
--!          die roten Leds herausgefuehrt.
--!          - @b control : rote LEDs 6..0
--!          - @b speedstep : rote LEDs 11..8
--! @details Die Schrittmotorsignale 4 Stepsignale und die beiden 
--!          Steuersignale werden auf die gruenen Leds herausgefuehrt
--!          - @b Schrittsignale : gruene LEDs 3..0
--!          - @b Steuersignale  : gruene LEDs 5..4
--! @details
--! @par History:
--! @details V1.0 16.12.2010 Ferdinand Englberger
--!          - Start der Entwicklung
-----------------------------------------------------------------------------

--! Use Standard Library
LIBRARY ieee; 
--! Use Logic Elements
USE ieee.std_logic_1164.all;
--! Use Conversion Functions
USE ieee.std_logic_arith.all;
--! Use Conversion Functions
USE ieee.std_logic_signed.all;

--! @brief Stepper Motor with Avalon Interface without
--!        Incremental Encoder
entity Stepper_IF is
	port
	(
		--! @name AVALON ports
		clock	   : in  STD_LOGIC;    --! AVALON Clock
		reset_n	   : in  STD_LOGIC; --! AVALON Reset
		ce_n       : in  STD_LOGIC;  --! Avalon Chip Enable
		read_n     : in  STD_LOGIC;  --! Avalon Read
		write_n    : in  STD_LOGIC;  --! Avalon Write
		addr       : in  STD_LOGIC_VECTOR (1 DOWNTO 0);  --! Avalon Adressbus
		write_data : in  STD_LOGIC_VECTOR (31 DOWNTO 0); --! Avalon Writebus
		read_data  : out STD_LOGIC_VECTOR (31 DOWNTO 0); --! Avalon Readbus       
      irq        : out STD_LOGIC; --! Avalon IRQ-Line
      --! @name Ports nach Aussen
      step_motor    : out STD_LOGIC_VECTOR (3 DOWNTO 0); --! externer Port: Schrittsignale
		motor_control : out STD_LOGIC_VECTOR (1 DOWNTO 0); --! externer Port: Motorsteuersignale
		greenleds     : out STD_LOGIC_VECTOR (5 DOWNTO 0); --! externer Port: gruene LEDs
      redleds       : out STD_LOGIC_VECTOR (11 DOWNTO 0) --! externer Port: rote LEDs
	);
end Stepper_IF;

--! @brief Architecture of Stepper Interface
--! @details Folgende Komponenten werden  realisiert:
--!          - drei Register fuer Motoransteuerung
architecture myStepInterface of Stepper_IF is
   -- Register
	signal control_reg : STD_LOGIC_VECTOR ( 6 DOWNTO 0);
	signal speed_reg   : STD_LOGIC_VECTOR ( 3 DOWNTO 0);
	signal steps_reg   : STD_LOGIC_VECTOR (13 DOWNTO 0);
	-- internal signals
	signal step_motor_int    : STD_LOGIC_VECTOR ( 3 DOWNTO 0);
	signal motor_control_int : STD_LOGIC_VECTOR ( 1 DOWNTO 0);
begin
    --! @brief Prozess fuer folgende Aufgaben:
	 --!        - Schnittstelle zum NIOS-Prozessor
    process (clock, reset_n, ce_n, read_n, write_n, addr, 
             write_data, control_reg, speed_reg, 
             steps_reg)
    begin

     -- Control Register  Write
     if (reset_n = '0') THEN
         control_reg <= B"0000000";
      elsif (clock'event and clock = '1' ) THEN
	     if (addr = B"00" AND write_n = '0' AND ce_n = '0') THEN
            control_reg <= write_data(6 DOWNTO 0);
	     else
	        control_reg <= control_reg;
	     end if;
	  end if;
	  
	  -- Speed Register  Write
     if (reset_n = '0') THEN
         speed_reg <= B"0000";
      elsif (clock'event and clock = '1' ) THEN
	     if (addr = B"01" AND write_n = '0' AND ce_n = '0') THEN
            speed_reg <= write_data(3 DOWNTO 0);
	     else
	        speed_reg <= speed_reg;
	     end if;
	  end if;
	
	  -- StepChain Register  Write
     if (reset_n = '0') THEN
         steps_reg <= B"00000000000000";
      elsif (clock'event and clock = '1' ) THEN
	     if (addr = B"10" AND write_n = '0' AND ce_n = '0') THEN
            steps_reg <= write_data(13 DOWNTO 0);
	     else
	        steps_reg <= steps_reg;
	     end if;
	  end if;
	
      -- Prozessor liest vom Register
      read_data <= (OTHERS => '0'); -- unused bits to 0
      if (read_n = '0'AND ce_n = '0')  THEN  
         case addr is
	       when B"00" =>
		        read_data(6 DOWNTO 0)  <= control_reg;           
	       when B"10" =>
	             read_data(3 DOWNTO 0)  <= speed_reg;
	       when B"11" =>
	             read_data(13 DOWNTO 0)  <= steps_reg;	      	
	       when others =>
	             read_data(6 DOWNTO 0)  <= control_reg;      
		     end case;
       end if;
       -- IRQ-Signal
       irq <= (control_reg(6) AND control_reg(5));       
    end process; 
	 
    -- Schnittstelle zu LEDS
    redleds(6 DOWNTO 0) <= control_reg;
	 redleds(7) <= '0';
	 redleds(11 DOWNTO 8) <= speed_reg;
	 
	 step_motor   <= step_motor_int;
	 motor_control <= motor_control_int;
	 greenleds(3 DOWNTO 0) <= step_motor_int;
	 greenleds(5 DOWNTO 4) <= motor_control_int;
	 -- Dummmy Zuweisung muss angepasst werden
	 step_motor_int    <= steps_reg(3 DOWNTO 0);
	 motor_control_int <= steps_reg(5 DOWNTO 4);
end myStepInterface;
